"""
Module containing functions to operate on files or directories.
"""

import pathlib


def get_svgs(path: pathlib.Path):
    """
    Find all SVGs in the given path.
    """
    if not path.exists():
        raise ValueError("Given path does not exist.")
    if not path.is_dir():
        return [path]
    
    return list(path.glob("**/*.svg"))


__all__ = (
    "get_svgs",
)



"""
Module containing the functions used to generate ``<svg>`` icons from other ``<svg>`` :class:`bs4.Tag`.
"""

import bs4


def compose_basic(background: bs4.Tag, icon: bs4.Tag) -> bs4.Tag:
    """
    Create a new and nice ``<svg>`` icon from the given background ``<svg>`` and the given foreground ``<svg>``.
    """

    if background.name != "svg":
        raise ValueError("bg is not a <svg> tag.")
    if icon.name != "svg":
        raise ValueError("fg is not a <svg> tag.")
        
    background = background.__copy__()
    background.attrs["id"] = "emblematic-background"
    background.attrs["width"] = "100%"
    background.attrs["height"] = "100%"
    
    icon = icon.__copy__()
    icon.attrs["id"] = "emblematic-icon"
    icon.attrs["width"] = "63%"
    icon.attrs["height"] = "63%"
    icon.attrs["preserveAspectRatio"] = "xMidYMid meet"
    icon.attrs["transform"] = "translate(370, 370)"    
    
    doc = bs4.BeautifulSoup("""
    <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 2000 2000">
    </svg>
    """, features="lxml-xml")
    container: bs4.Tag = doc.svg
    container.append(background)
    container.append(icon)

    return doc
    

__all__ = (
    "compose_basic",
)



"""
Command-line interface for :mod:`emblematic`.

Implemented with :mod:`click`.
"""

import click
import bs4
import pathlib
import cairosvg

from .files import get_svgs
from .compose import compose_basic


@click.group()
def main():
    pass


@main.command("basic")
@click.option(
    "-b", "--background", "bg_file",
    type=click.File(mode="r"),
    required=True,
    help="SVG file to be used as background.",
)
@click.option(
    "-i", "--icon", "icon_paths",
    type=click.Path(exists=True),
    required=True,
    multiple=True,
    help="SVG files or directories of files to be used as foreground.",
)
@click.option(
    "-f", "--fill", "icon_fill",
    type=str,
    required=True,
    help="The color to fill icons with."
)
@click.option(
    "-o", "--output-dir", "output_dir",
    type=click.Path(exists=True, file_okay=False, dir_okay=True),
    required=True,
    help="The directory where output files should be placed in."
)
def basic(bg_file, icon_paths, icon_fill, output_dir):
    icon_paths = map(pathlib.Path, icon_paths)
    icon_paths = map(get_svgs, icon_paths)
    icon_paths = sum(icon_paths, start=[])

    output_dir = pathlib.Path(output_dir)

    bg_doc = bs4.BeautifulSoup(bg_file, features="lxml-xml")
    bg = bg_doc.svg

    for icon_path in icon_paths:
        icon_path: pathlib.Path
        output_path = output_dir.joinpath(f"{icon_path.stem}.png")

        with open(icon_path) as icon_file:
            click.echo(icon_path, nl=False)
            icon_doc = bs4.BeautifulSoup(icon_file, features="lxml-xml")
            icon = icon_doc.svg
            icon.path.attrs["fill"] = icon_fill
        
        click.echo(" → ", nl=False)
        svg_doc = compose_basic(background=bg, icon=icon)

        click.echo(" → ", nl=False)
        svg_bytes = bytes(svg_doc.prettify(), encoding="utf8")
        
        click.echo(" → ", nl=False)
        png_bytes = cairosvg.svg2png(bytestring=svg_bytes)

        with open(output_path, mode="wb") as output_file:
            click.echo(output_path, nl=False)
            output_file.write(png_bytes)

        click.echo()



if __name__ == "__main__":
    main()
